@@ -22,7 +22,7 @@ import sys
 sys.path[0:0] = [""]
 
 from bson.objectid import ObjectId
-from bson.py3compat import u, StringIO
+from bson.py3compat import StringIO
 from gridfs import GridFS
 from gridfs.grid_file import (DEFAULT_CHUNK_SIZE,
                               _SEEK_CUR,
@@ -34,6 +34,7 @@ from gridfs.errors import NoFile
 from pymongo import MongoClient
 from pymongo.errors import ConfigurationError, ServerSelectionTimeoutError
 from test import (IntegrationTest,
+                  client_context,
                   host,
                   port,
                   unittest,
@@ -44,6 +45,7 @@ from test.utils import rs_or_single_client
 class TestGridFileNoConnect(unittest.TestCase):
 
     @classmethod
+    @client_context.require_connection
     def setUpClass(cls):
         client = MongoClient(host, port, connect=False)
         cls.db = client.pymongo_test
@@ -498,21 +500,21 @@ Bye"""))
 
     def test_write_unicode(self):
         f = GridIn(self.db.fs)
-        self.assertRaises(TypeError, f.write, u("foo"))
+        self.assertRaises(TypeError, f.write, u"foo")
 
         f = GridIn(self.db.fs, encoding="utf-8")
-        f.write(u("foo"))
+        f.write(u"foo")
         f.close()
 
         g = GridOut(self.db.fs, f._id)
         self.assertEqual(b"foo", g.read())
 
         f = GridIn(self.db.fs, encoding="iso-8859-1")
-        f.write(u("aé"))
+        f.write(u"aé")
         f.close()
 
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(u("aé").encode("iso-8859-1"), g.read())
+        self.assertEqual(u"aé".encode("iso-8859-1"), g.read())
 
     def test_set_after_close(self):
         f = GridIn(self.db.fs, _id="foo", bar="baz")
