@@ -25,7 +25,7 @@ import time
 import gridfs
 
 from bson.binary import Binary
-from bson.py3compat import u, StringIO, string_type
+from bson.py3compat import StringIO, string_type
 from pymongo.mongo_client import MongoClient
 from pymongo.errors import (ConfigurationError,
                             ConnectionFailure,
@@ -371,14 +371,14 @@ class TestGridfs(IntegrationTest):
         self.assertFalse(self.fs.exists({"foo": {"$gt": 12}}))
 
     def test_put_unicode(self):
-        self.assertRaises(TypeError, self.fs.put, u("hello"))
+        self.assertRaises(TypeError, self.fs.put, u"hello")
 
-        oid = self.fs.put(u("hello"), encoding="utf-8")
+        oid = self.fs.put(u"hello", encoding="utf-8")
         self.assertEqual(b"hello", self.fs.get(oid).read())
         self.assertEqual("utf-8", self.fs.get(oid).encoding)
 
-        oid = self.fs.put(u("aé"), encoding="iso-8859-1")
-        self.assertEqual(u("aé").encode("iso-8859-1"), self.fs.get(oid).read())
+        oid = self.fs.put(u"aé", encoding="iso-8859-1")
+        self.assertEqual(u"aé".encode("iso-8859-1"), self.fs.get(oid).read())
         self.assertEqual("iso-8859-1", self.fs.get(oid).encoding)
 
     def test_missing_length_iter(self):
@@ -468,6 +468,10 @@ class TestGridfsReplicaSet(TestReplicaSetClientBase):
             read_preference=ReadPreference.SECONDARY)
 
         fs = gridfs.GridFS(rsc.pymongo_test)
+
+        gin = fs.new_file()
+        self.assertEqual(gin._coll.read_preference, ReadPreference.PRIMARY)
+
         oid = fs.put(b'foo')
         content = fs.get(oid).read()
         self.assertEqual(b'foo', content)
